/******************************************************************************/
/* APlayer global header file.                                                */
/******************************************************************************/
/* This is part of the APlayer Programming Package (APPP).                    */
/* Copyright (C) 1998-2000 by The APlayer-Team.                               */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may only be used in APlayer related         */
/* software. If you want to use it elsewhere, please contact the author for a */
/* permission.                                                                */
/******************************************************************************/


#ifndef __APlayer_h
#define __APlayer_h

// PolyKit headers
#include "POS.h"
#include "PString.h"
#include "PRegistry.h"
#include "PSynchronize.h"

// APlayer headers
#include "APList.h"
#include "APChannel.h"
#include "APAddOns.h"
#include "APlayerClasses_Import.h"


// Constants for the layout
#if __p_os == __p_beos

#define HSPACE		4.0
#define VSPACE		4.0

#define PICHSIZE	16.0
#define PICVSIZE	16.0

#define OUTPUTAGENT_MAX_VIEW_WIDTH	524.0
#define OUTPUTAGENT_MAX_VIEW_HEIGHT	93.0

#elif __p_os == __p_windows

#define HSPACE		4
#define VSPACE		4

#define PICHSIZE	16
#define PICVSIZE	16

#define OUTPUTAGENT_MAX_VIEW_WIDTH	524
#define OUTPUTAGENT_MAX_VIEW_HEIGHT	93

#endif


// Private functions
typedef APAddOnConverter *(*AllocateConverterFunc)(void *converter);



/******************************************************************************/
/* Class given to the player and agents constructor.                          */
/* Remember to call Lock() and Unlock() when you accessing the data.          */
/******************************************************************************/
class APAddOnPlayer;
class APModuleList;

#if __p_os == __p_beos && __POWERPC__
#pragma export on
#endif

class _IMPEXP_APLIB APGlobalData : public PMutex
{
public:
	APGlobalData(void);
	virtual ~APGlobalData(void);

	void InstallModType(PString type, PString extension, PString lDescription, PString sDescription, void *smallIcon, void *largeIcon);

	void CallAfterAddingFilesPlugins(void);
	void CallAfterRemovingFilesPlugins(void);

	void GetLoaderConverterNames(PList<PString> &names);
	void GetSaverConverterNames(PList<PString> &names);
	APAddOnConverter *GetConverterInstance(PString name);
	void DeleteConverterInstance(APAddOnConverter *converter);

#if __p_os == __p_beos

	bool ShowConverterConfig(APAddOnConverter *converter, BView *view);

#elif __p_os == __p_windows

	bool ShowConverterConfig(APAddOnConverter *converter, CView *view);

#endif

	PRegistry *		useReg;				// Read and write your config data to this one
	PRegistry *		saveReg;			// Write only your config data to this one

	APModuleList *	moduleList;			// Use this class to work on module lists

	APAddOnPlayer *	player;				// A pointer to the player in use
	uint32			playerIndex;		// Is the index number in the player add-on
	PString			playerName;			// The name of the player

	PString			moduleFormat;		// The original module format

	PString			fileName;			// This is the filename with full path
	uint32			moduleLen;			// The length of the module in bytes

	PString			author;				// The author of the module

#if __p_os == __p_windows
#pragma warning(disable:4251)
#endif

	PList<APInstInfo *> instruments;	// Read only from this list. It contains the instrument data of the current playing module
	PList<APSampleInfo *> samples;		// Read only from this list. It contains the sample data of the current playing module

protected:
	APPriList<APAddOnAgent *> afterAddingFilesAgents;
	APPriList<APAddOnAgent *> afterRemovingFilesAgents;

	PList<void *> converterLoaders;
	PList<void *> converterSavers;

	APList<void *> addonFileTypeList;

#if __p_os == __p_windows
#pragma warning(default:4251)
#endif

	AllocateConverterFunc allocateConverter;
};

#if __p_os == __p_beos && __POWERPC__
#pragma export off
#endif

#endif
